@@ -1,5 +1,4 @@
 """
-文档中示例代码在线运行
 Run the example code in the documentation online
 """
 from functools import partial
@@ -8,6 +7,7 @@ from os import path, listdir
 from pywebio import start_server
 from pywebio.output import *
 from pywebio.input import *
+from pywebio.pin import *
 from pywebio.platform import seo
 from pywebio.session import *
 from pywebio.session import local as session_local
@@ -47,6 +47,7 @@ def run_code(code, scope):
 IMPORT_CODE = """from pywebio.input import *
 from pywebio.output import *
 from pywebio.session import *
+from pywebio.pin import *
 
 """
 
