@@ -6,13 +6,31 @@ import json
 import logging
 from collections import UserList
 from functools import partial, wraps
-
+from collections.abc import Mapping
 from .session import chose_impl, next_client_event, get_current_task_id, get_current_session
 from .utils import random_str
 
 logger = logging.getLogger(__name__)
 
 
+def scope2dom(name, no_css_selector=False):
+    """Get the CSS selector/element name actually used in the front-end html page
+
+    :param str/tuple name: When it is str, it is regarded as the Dom ID name;
+       when tuple, the format is (css selector, element name)
+    """
+    selector = '#'
+    if isinstance(name, tuple):
+        selector, name = name
+
+    name = name.replace(' ', '-')
+
+    if no_css_selector:
+        selector = ''
+
+    return '%spywebio-scope-%s' % (selector, name)
+
+
 class Output:
     """ ``put_xxx()`` 类函数的返回值
 
@@ -73,12 +91,12 @@ class Output:
     def __enter__(self):
         if not self.enabled_context_manager:
             raise RuntimeError("This output function can't be used as context manager!")
-        r = self.custom_enter(self) if self.custom_enter else None
-        if r is not None:
-            return r
+        if self.custom_enter:
+            return self.custom_enter(self)
+
         self.container_dom_id = self.container_dom_id or random_str(10)
         self.spec['container_selector'] = self.container_selector
-        self.spec['container_dom_id'] = self.container_dom_id
+        self.spec['container_dom_id'] = scope2dom(self.container_dom_id, no_css_selector=True)
         self.send()
         get_current_session().push_scope(self.container_dom_id)
         return self.container_dom_id
@@ -89,9 +107,9 @@ class Output:
         it means that the context manager can handle the exception,
         so that the with statement terminates the propagation of the exception
         """
-        r = self.custom_exit(self, exc_type=exc_type, exc_val=exc_val, exc_tb=exc_tb) if self.custom_exit else None
-        if r is not None:
-            return r
+        if self.custom_exit:
+            return self.custom_exit(self, exc_type=exc_type, exc_val=exc_val, exc_tb=exc_tb)
+
         get_current_session().pop_scope()
         return False  # Propagate Exception
 
@@ -105,6 +123,25 @@ class Output:
         self.processed = True
         send_msg('output', self.spec)
 
+    show = send  # `show` is a more user-friendly name
+
+    def style(self, css_style):
+        """Set css style for output
+
+        Example::
+
+            put_text('hello').style('color: red; font-size: 20px')
+
+            put_row([
+                put_text('hello').style('color: red'),
+                put_markdown('markdown')
+            ]).style('margin-top: 20px')
+
+        """
+        self.spec.setdefault('style', '')
+        self.spec['style'] += ';%s' % css_style
+        return self
+
     def __del__(self):
         """返回值没有被变量接收时的操作：直接输出消息"""
         if not self.processed:
@@ -152,24 +189,42 @@ def safely_destruct_output_when_exp(content_param):
     return decorator
 
 
-def send_msg(cmd, spec=None):
-    msg = dict(command=cmd, spec=spec, task_id=get_current_task_id())
+def send_msg(cmd, spec=None, task_id=None):
+    msg = dict(command=cmd, spec=spec, task_id=task_id or get_current_task_id())
     get_current_session().send_task_command(msg)
 
 
+def single_input_kwargs(single_input_return):
+    try:
+        # 协程模式下，单项输入为协程对象，可以通过send(None)来获取传入单项输入的参数字典
+        # In the coroutine mode, the item of `inputs` is coroutine object.
+        # using `send(None)` to get the single input function's parameter dict.
+        single_input_return.send(None)
+    except StopIteration as e:  # This is in the coroutine mode
+        input_kwargs = e.args[0]
+    except AttributeError:  # This is in the thread mode
+        input_kwargs = single_input_return
+    else:
+        raise RuntimeError("Can't get kwargs from single input")
+    return input_kwargs
+
+
 @chose_impl
-def single_input(item_spec, valid_func, preprocess_func):
+def single_input(item_spec, valid_func, preprocess_func, onchange_func):
     """
     Note: 鲁棒性在上层完成
     将单个input构造成input_group，并获取返回值
     :param item_spec: 单个输入项的参数 'name' must in item_spec， 参数一定已经验证通过
     :param valid_func: Not None
+    :param onchange_func: Not None
     :param preprocess_func: Not None, 预处理函数，在收到用户提交的单项输入的原始数据后用于在校验前对数据进行预处理
     """
     if item_spec.get('name') is None:  # single input
         item_spec['name'] = 'data'
     else:  # as input_group item
-        return dict(item_spec=item_spec, valid_func=valid_func, preprocess_func=preprocess_func)
+        # use `single_input_kwargs()` to get the returned value
+        return dict(item_spec=item_spec, valid_func=valid_func,
+                    preprocess_func=preprocess_func, onchange_func=onchange_func)
 
     label = item_spec['label']
     name = item_spec['name']
@@ -179,23 +234,27 @@ def single_input(item_spec, valid_func, preprocess_func):
     item_spec.setdefault('auto_focus', True)  # 如果没有设置autofocus参数，则开启参数  todo CHECKBOX, RADIO 特殊处理
 
     spec = dict(label=label, inputs=[item_spec])
-    data = yield input_control(spec, {name: preprocess_func}, {name: valid_func})
+    data = yield input_control(spec=spec,
+                               preprocess_funcs={name: preprocess_func},
+                               item_valid_funcs={name: valid_func},
+                               onchange_funcs={name: onchange_func})
     return data[name]
 
 
 @chose_impl
-def input_control(spec, preprocess_funcs, item_valid_funcs, form_valid_funcs=None):
+def input_control(spec, preprocess_funcs, item_valid_funcs, onchange_funcs, form_valid_funcs=None):
     """
     发送input命令，监听事件，验证输入项，返回结果
     :param spec:
     :param preprocess_funcs: keys 严格等于 spec中的name集合
     :param item_valid_funcs: keys 严格等于 spec中的name集合
-    :param form_valid_funcs:
+    :param onchange_funcs: keys 严格等于 spec中的name集合
+    :param form_valid_funcs: can be ``None``
     :return:
     """
     send_msg('input_group', spec)
 
-    data = yield input_event_handle(item_valid_funcs, form_valid_funcs, preprocess_funcs)
+    data = yield input_event_handle(item_valid_funcs, form_valid_funcs, preprocess_funcs, onchange_funcs)
 
     send_msg('destroy_form')
     return data
@@ -223,14 +282,29 @@ def check_item(name, data, valid_func, preprocess_func):
     return True
 
 
+def trigger_onchange(event_data, onchange_funcs):
+    name = event_data['name']
+    onchange_func = onchange_funcs[name]
+
+    task_id = get_current_task_id()
+    get_current_session().internal_save['onchange_trigger-' + task_id] = name  # used in `pywebio.input.input_update()`
+    try:
+        onchange_func(event_data['value'])
+    except Exception as e:
+        logger.warning('Get %r in onchange function for name:"%s"', e, name)
+    finally:
+        del get_current_session().internal_save['onchange_trigger-' + task_id]
+
+
 @chose_impl
-def input_event_handle(item_valid_funcs, form_valid_funcs, preprocess_funcs):
+def input_event_handle(item_valid_funcs, form_valid_funcs, preprocess_funcs, onchange_funcs):
     """
     根据提供的校验函数处理表单事件
     :param item_valid_funcs: map(name -> valid_func)  valid_func 为 None 时，不进行验证
                         valid_func: callback(data) -> error_msg or None
     :param form_valid_funcs: callback(data) -> (name, error_msg) or None
-    :param preprocess_funcs:
+    :param preprocess_funcs: map(name -> process_func)
+    :param onchange_funcs: map(name -> onchange_func)
     :return:
     """
     while True:
@@ -242,6 +316,8 @@ def input_event_handle(item_valid_funcs, form_valid_funcs, preprocess_funcs):
                 onblur_name = event_data['name']
                 check_item(onblur_name, event_data['value'], item_valid_funcs[onblur_name],
                            preprocess_funcs[onblur_name])
+            elif input_event == 'change':
+                trigger_onchange(event_data, onchange_funcs)
 
         elif event_name == 'from_submit':
             all_valid = True
