@@ -58,6 +58,8 @@ Functions list
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `put_file`                | Output a link to download a file                           |
 |                    +---------------------------+------------------------------------------------------------+
+|                    | `put_tabs`:sup:`*`        | Output tabs                                                |
+|                    +---------------------------+------------------------------------------------------------+
 |                    | `put_collapse`:sup:`*†`   | Output collapsible content                                 |
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `put_scrollable`:sup:`*†` | | Output a fixed height content area,                      |
@@ -124,6 +126,7 @@ Content Outputting
 .. autofunction:: put_buttons
 .. autofunction:: put_image
 .. autofunction:: put_file
+.. autofunction:: put_tabs
 .. autofunction:: put_collapse
 .. autofunction:: put_scrollable
 .. autofunction:: put_widget
@@ -157,7 +160,7 @@ from collections.abc import Mapping, Sequence
 from functools import wraps
 from typing import Union
 
-from .io_ctrl import output_register_callback, send_msg, Output, safely_destruct_output_when_exp, OutputList
+from .io_ctrl import output_register_callback, send_msg, Output, safely_destruct_output_when_exp, OutputList, scope2dom
 from .session import get_current_session, download
 from .utils import random_str, iscoroutinefunction, is_html_safe_value
 
@@ -168,7 +171,7 @@ except ImportError:
 
 logger = logging.getLogger(__name__)
 
-__all__ = ['Position', 'remove', 'scroll_to',
+__all__ = ['Position', 'remove', 'scroll_to', 'put_tabs',
            'put_text', 'put_html', 'put_code', 'put_markdown', 'use_scope', 'set_scope', 'clear', 'remove',
            'put_table', 'put_buttons', 'put_image', 'put_file', 'PopupSize', 'popup',
            'close_popup', 'put_widget', 'put_collapse', 'put_link', 'put_scrollable', 'style', 'put_column',
@@ -204,22 +207,6 @@ class Scope:
 _scope_name_allowed_chars = set(string.ascii_letters + string.digits + '_-')
 
 
-def _parse_scope(name, no_css_selector=False):
-    """Get the CSS selector/element name actually used in the front-end html page
-
-    :param str/tuple name: When it is str, it is regarded as the Dom ID name;
-       when tuple, the format is (css selector, element name)
-    """
-    selector = '#'
-    if isinstance(name, tuple):
-        selector, name = name
-
-    name = name.replace(' ', '-')
-
-    if no_css_selector:
-        selector = ''
-
-    return '%spywebio-scope-%s' % (selector, name)
 
 
 def set_scope(name, container_scope=Scope.Current, position=OutputPosition.BOTTOM, if_exist=None):
@@ -242,8 +229,8 @@ def set_scope(name, container_scope=Scope.Current, position=OutputPosition.BOTTO
         container_scope = get_current_session().get_scope_name(container_scope)
 
     assert is_html_safe_value(name), "Scope name only allow letter/digit/'_'/'-' char."
-    send_msg('output_ctl', dict(set_scope=_parse_scope(name, no_css_selector=True),
-                                container=_parse_scope(container_scope),
+    send_msg('output_ctl', dict(set_scope=scope2dom(name, no_css_selector=True),
+                                container=scope2dom(container_scope),
                                 position=position, if_exist=if_exist))
 
 
@@ -270,7 +257,7 @@ def clear(scope=Scope.Current):
     """
     if isinstance(scope, int):
         scope = get_current_session().get_scope_name(scope)
-    send_msg('output_ctl', dict(clear=_parse_scope(scope)))
+    send_msg('output_ctl', dict(clear=scope2dom(scope)))
 
 
 def remove(scope=Scope.Current):
@@ -281,7 +268,7 @@ def remove(scope=Scope.Current):
     if isinstance(scope, int):
         scope = get_current_session().get_scope_name(scope)
     assert scope != 'ROOT', "Can not remove `ROOT` scope."
-    send_msg('output_ctl', dict(remove=_parse_scope(scope)))
+    send_msg('output_ctl', dict(remove=scope2dom(scope)))
 
 
 def scroll_to(scope=Scope.Current, position=Position.TOP):
@@ -297,7 +284,7 @@ def scroll_to(scope=Scope.Current, position=Position.TOP):
     """
     if isinstance(scope, int):
         scope = get_current_session().get_scope_name(scope)
-    send_msg('output_ctl', dict(scroll_to=_parse_scope(scope), position=position))
+    send_msg('output_ctl', dict(scroll_to=scope2dom(scope), position=position))
 
 
 def _get_output_spec(type, scope, position, **other_spec):
@@ -321,7 +308,7 @@ def _get_output_spec(type, scope, position, **other_spec):
     else:
         scope_name = scope
 
-    spec['scope'] = _parse_scope(scope_name)
+    spec['scope'] = scope2dom(scope_name)
     spec['position'] = position
 
     return spec
@@ -406,8 +393,13 @@ def put_html(html, sanitize=False, scope=Scope.Current, position=OutputPosition.
     :param bool sanitize: Whether to use `DOMPurify <https://github.com/cure53/DOMPurify>`_ to filter the content to prevent XSS attacks.
     :param int scope, position: Those arguments have the same meaning as for `put_text()`
     """
+
+    # Compatible with ipython rich output
+    # See: https://ipython.readthedocs.io/en/stable/config/integrating.html?highlight=Rich%20display#rich-display
     if hasattr(html, '__html__'):
         html = html.__html__()
+    elif hasattr(html, '_repr_html_'):
+        html = html._repr_html_()
 
     spec = _get_output_spec('html', content=html, sanitize=sanitize, scope=scope, position=position)
     return Output(spec)
@@ -636,8 +628,8 @@ def _format_button(buttons):
     return btns
 
 
-def put_buttons(buttons, onclick, small=None, link_style=False, scope=Scope.Current, position=OutputPosition.BOTTOM,
-                **callback_options) -> Output:
+def put_buttons(buttons, onclick, small=None, link_style=False, outline=False, group=False, scope=Scope.Current,
+                position=OutputPosition.BOTTOM, **callback_options) -> Output:
     """
     Output a group of buttons and bind click event
 
@@ -656,10 +648,10 @@ def put_buttons(buttons, onclick, small=None, link_style=False, scope=Scope.Curr
             :name: put_buttons-btn_class
             :summary: `put_buttons()`
 
-            put_buttons([dict(label='primary', value='p', color='primary')], onclick=...)  # ..doc-only
+            put_buttons([dict(label='success', value='s', color='success')], onclick=...)  # ..doc-only
             put_buttons([  # ..demo-only
                 dict(label=i, value=i, color=i)  # ..demo-only
-                for i in ['primary' , 'secondary' , 'success' , 'danger' , 'warning' , 'info' , 'light' , 'dark']  # ..demo-only
+                for i in ['primary', 'secondary', 'success', 'danger', 'warning', 'info', 'light', 'dark']  # ..demo-only
             ], onclick=put_text)  # ..demo-only
 
     :type onclick: callable / list
@@ -674,6 +666,8 @@ def put_buttons(buttons, onclick, small=None, link_style=False, scope=Scope.Curr
        Note: When in :ref:`Coroutine-based session  <coroutine_based_session>`, the callback can be a coroutine function.
     :param bool small: Whether to use small size button. Default is False.
     :param bool link_style: Whether to use link style button. Default is False
+    :param bool outline: Whether to use outline style button. Default is False
+    :param bool group: Whether to group the buttons together. Default is False
     :param int scope, position: Those arguments have the same meaning as for `put_text()`
     :param callback_options: Other options of the ``onclick`` callback. There are different options according to the session implementation
 
@@ -719,14 +713,13 @@ def put_buttons(buttons, onclick, small=None, link_style=False, scope=Scope.Curr
 
     def click_callback(btn_val):
         if isinstance(onclick, dict):
-            func = onclick.get(btn_val, lambda: None)
-            return func()
+            return onclick[btn_val]()
         else:
             return onclick(btn_val)
 
     callback_id = output_register_callback(click_callback, **callback_options)
     spec = _get_output_spec('buttons', callback_id=callback_id, buttons=btns, small=small,
-                            scope=scope, position=position, link=link_style)
+                            scope=scope, position=position, link=link_style, outline=outline, group=group)
 
     return Output(spec)
 
@@ -758,7 +751,7 @@ def put_image(src, format=None, title='', width=None, height=None,
         put_image('https://www.python.org/static/img/python-logo.png')
     """
     if isinstance(src, PILImage):
-        format = src.format
+        format = format or src.format or 'JPEG'
         imgByteArr = io.BytesIO()
         src.save(imgByteArr, format=format)
         src = imgByteArr.getvalue()
@@ -1072,6 +1065,41 @@ def put_scrollable(content=[], height=400, keep_bottom=False, horizon_scroll=Fal
                       scope=scope, position=position).enable_context_manager()
 
 
+@safely_destruct_output_when_exp('tabs')
+def put_tabs(tabs, scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
+    """Output tabs.
+
+    :param list tabs: Tab list, each item is a dict: ``{"title": "Title", "content": ...}`` .
+       The ``content`` can be a string, the ``put_xxx()`` calls , or a list of them.
+    :param int scope, position: Those arguments have the same meaning as for `put_text()`
+
+    .. exportable-codeblock::
+        :name: put_tabs
+        :summary: `put_tabs()` usage
+
+        put_tabs([
+            {'title': 'Text', 'content': 'Hello world'},
+            {'title': 'Markdown', 'content': put_markdown('~~Strikethrough~~')},
+            {'title': 'More content', 'content': [
+                put_table([
+                    ['Commodity', 'Price'],
+                    ['Apple', '5.5'],
+                    ['Banana', '7'],
+                ]),
+                put_link('pywebio', 'https://github.com/wang0618/PyWebIO')
+            ]},
+        ])
+
+    .. versionadded:: 1.3
+    """
+
+    for tab in tabs:
+        assert 'title' in tab and 'content' in tab
+
+    spec = _get_output_spec('tabs', tabs=tabs, scope=scope, position=position)
+    return Output(spec)
+
+
 @safely_destruct_output_when_exp('data')
 def put_widget(template, data, scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
     """Output your own widget
@@ -1088,7 +1116,7 @@ def put_widget(template, data, scope=Scope.Current, position=OutputPosition.BOTT
 
     .. exportable-codeblock::
         :name: put_widget
-        :summary: Use `put_widget()`to output your own widget
+        :summary: Use `put_widget()` to output your own widget
 
         tpl = '''
         <details {{#open}}open{{/open}}>
@@ -1338,7 +1366,7 @@ def output(*contents):
             for o in outputs:
                 if not isinstance(o, Output):
                     o = put_text(o)
-                o.spec['scope'] = _parse_scope(self.scope)
+                o.spec['scope'] = scope2dom(self.scope)
                 o.spec['position'] = OutputPosition.BOTTOM
                 o.send()
 
@@ -1352,7 +1380,7 @@ def output(*contents):
             for acc, o in enumerate(outputs):
                 if not isinstance(o, Output):
                     o = put_text(o)
-                o.spec['scope'] = _parse_scope(self.scope)
+                o.spec['scope'] = scope2dom(self.scope)
                 o.spec['position'] = idx + direction * acc
                 o.send()
 
@@ -1367,7 +1395,7 @@ def output(*contents):
             {{/contents}}
         </div>"""
     out_spec = put_widget(template=tpl,
-                          data=dict(contents=contents, dom_class_name=_parse_scope(dom_name, no_css_selector=True)))
+                          data=dict(contents=contents, dom_class_name=scope2dom(dom_name, no_css_selector=True)))
     return OutputHandler(Output.dump_dict(out_spec), ('.', dom_name))
 
 
@@ -1375,6 +1403,9 @@ def output(*contents):
 def style(outputs, css_style) -> Union[Output, OutputList]:
     """Customize the css style of output content
 
+    .. deprecated:: 1.3
+        See :ref:`User Guide <style>` for new way to set css style for output.
+
     :param outputs: The output content can be a ``put_xxx()`` call or a list of it.
     :type outputs: list/put_xxx()
     :param str css_style: css style string
@@ -1385,7 +1416,7 @@ def style(outputs, css_style) -> Union[Output, OutputList]:
     :Example:
 
     .. exportable-codeblock::
-        :name: style
+        :name: style-deprecated
         :summary: `style()` usage
 
         style(put_text('Red'), 'color:red')
@@ -1409,6 +1440,10 @@ def style(outputs, css_style) -> Union[Output, OutputList]:
         ], 'margin-left:20px'))
 
     """
+    import warnings
+    warnings.warn("`pywebio.output.style()` is deprecated since v1.3 and will remove in the future version, "
+                  "use `put_xxx(...).style(...)` instead", DeprecationWarning, stacklevel=2)
+
     if not isinstance(outputs, (list, tuple, OutputList)):
         ol = [outputs]
     else:
@@ -1501,7 +1536,7 @@ def popup(title, content=None, size=PopupSize.NORMAL, implicit_close=True, closa
 
     send_msg(cmd='popup', spec=dict(content=Output.dump_dict(content), title=title, size=size,
                                     implicit_close=implicit_close, closable=closable,
-                                    dom_id=_parse_scope(dom_id, no_css_selector=True)))
+                                    dom_id=scope2dom(dom_id, no_css_selector=True)))
 
     return use_scope_(dom_id)
 
