@@ -1,13 +1,49 @@
 import asyncio
 import re
 from collections.abc import Sequence
-from pywebio.session import get_info
+
 from pywebio.output import *
+from pywebio.session import get_info
+
+requirejs_config_tpl = """
+<script type="text/javascript">
+require.config({
+    paths: {
+        "bokeh": "https://cdn.jsdelivr.net/npm/@bokeh/bokehjs@__version__/build/js/bokeh.min",
+        "bokeh-widgets": "https://cdn.jsdelivr.net/npm/@bokeh/bokehjs@__version__/build/js/bokeh-widgets.min",
+        "bokeh-tables": "https://cdn.jsdelivr.net/npm/@bokeh/bokehjs@__version__/build/js/bokeh-tables.min",
+        "bokeh-api": "https://cdn.jsdelivr.net/npm/@bokeh/bokehjs@__version__/build/js/bokeh-api.min",
+        "bokeh-gl": "https://cdn.jsdelivr.net/npm/@bokeh/bokehjs@__version__/build/js/bokeh-gl.min",
+    },
+    shim: {
+        'bokeh': {
+            exports: 'Bokeh'
+        },
+        'bokeh-widgets': {
+            exports: '_',
+            deps: ['bokeh'],
+        },
+        'bokeh-tables': {
+            exports: '_',
+            deps: ['bokeh'],
+        },
+        'bokeh-api': {
+            exports: '_',
+            deps: ['bokeh'],
+        },
+        'bokeh-gl': {
+            exports: '_',
+            deps: ['bokeh'],
+        },
+    }
+});
+</script>
+"""
 
 requirejs_tpl = """
 %s
 <script type="text/javascript">
-requirejs(['bokeh', 'bokeh-widgets', 'bokeh-tables', 'bokeh-gl'], function(Bokeh) {
+requirejs(['bokeh', 'bokeh-widgets', 'bokeh-tables'], function(Bokeh) {
     %s
 });
 </script>
@@ -23,11 +59,12 @@ def load_notebook(resources=None, verbose=False, hide_banner=False, load_timeout
     :param load_timeout: 不支持
     :return: None
     """
+    from bokeh import __version__
     from bokeh.util.serialization import make_id
 
     js_gists = ["console.log('Load BokehJS complete.')"]
 
-    html = ''
+    html = requirejs_config_tpl.replace('__version__', __version__)
     if verbose:
         element_id = make_id()
         html += """
@@ -43,11 +80,11 @@ def load_notebook(resources=None, verbose=False, hide_banner=False, load_timeout
         js_gists.append('Bokeh.set_log_level("info");')
         js_gists.append("console.log('Set bokeh log level to INFO because you set `output_notebook(verbose=True)`')")
 
-    put_html(requirejs_tpl % (html, '\n'.join(js_gists)))
+    put_html(requirejs_tpl % (html, '\n'.join(js_gists)), sanitize=False)
 
 
 def show_doc(obj, state, notebook_handle):
-    """显示 Bokeh 单个 documents
+    """Show a document of Bokeh
 
     :param obj:
     :param state:
@@ -62,11 +99,11 @@ def show_doc(obj, state, notebook_handle):
     elif isinstance(obj, dict):
         div = '\n'.join(div[k] for k in obj.keys())
 
-    put_html(requirejs_tpl % (div, script))
+    put_html(requirejs_tpl % (div, script), sanitize=False)
 
 
 def show_app(app, state, notebook_url, port=0, **kw):
-    """显示 Bokeh applications
+    """Show Bokeh applications
 
     :param app: A Bokeh Application to embed in PyWebIO.
     :param state: ** Unused **
@@ -80,6 +117,10 @@ def show_app(app, state, notebook_url, port=0, **kw):
 
     from pywebio.platform.tornado import ioloop
     loop = ioloop()
+    if loop is None:
+        toast("Currently only supports showing bokeh application in Tornado backend",
+              color='error', duration=0)
+        return
     loop.make_current()
     asyncio.set_event_loop(loop.asyncio_loop)
     # loop = IOLoop.current()
@@ -104,13 +145,13 @@ def show_app(app, state, notebook_url, port=0, **kw):
 
     script = re.sub(r'<script(.*?)>([\s\S]*?)</script>', r"""
     <script \g<1>>
-        requirejs(['bokeh', 'bokeh-widgets', 'bokeh-tables', 'bokeh-gl'], function(Bokeh) {
+        requirejs(['bokeh', 'bokeh-widgets', 'bokeh-tables'], function(Bokeh) {
             \g<2>
         });
     </script>
     """, script)
 
-    put_html(script)
+    put_html(script, sanitize=False)
 
 
 def try_install_bokeh_hook():
