@@ -57,8 +57,8 @@ class Session:
         """
         :param session_info: 会话信息。可以通过 Session.info 访问
         """
-        self.info = session_info
-        self.save = {}
+        self.internal_save = dict(info=session_info)  # some session related info, just for internal used
+        self.save = {}  # underlying implement of `pywebio.session.data`
         self.scope_stack = defaultdict(lambda: ['ROOT'])  # task_id -> scope栈
 
         self.deferred_functions = []  # 会话结束时运行的函数
@@ -125,13 +125,19 @@ class Session:
     def on_task_exception(self):
         from ..output import toast
         from . import run_js
+        from . import info as session_info
+
         logger.exception('Error')
+
+        toast_msg = "应用发生内部错误" if 'zh' in session_info.user_language else "An internal error occurred in the application"
+
         type, value, tb = sys.exc_info()
         lines = traceback.format_exception(type, value, tb)
         traceback_msg = ''.join(lines)
         traceback_msg = 'Internal Server Error\n' + traceback_msg
+
         try:
-            toast('应用发生内部错误', duration=1, color='error')
+            toast(toast_msg, duration=1, color='error')
             run_js("console.error(traceback_msg)", traceback_msg=traceback_msg)
         except Exception:
             pass
