@@ -367,4 +367,13 @@ def parse_file_size(size):
             base = 2 ** (idx * 10)
             return int(float(s) * base)
 
-    return int(size)
\ No newline at end of file
+    return int(size)
+
+
+def strip_space(text, n):
+    """strip n spaces of every line in text"""
+    lines = (
+        i[n:] if (i[:n] == ' ' * n) else i
+        for i in text.splitlines()
+    )
+    return '\n'.join(lines)
